@@ -5,20 +5,22 @@ package query
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
 	"math"
 	"net"
 	"testing"
 	"time"
 
-	"github.com/fortytw2/leaktest"
+	"github.com/pkg/errors"
+	"google.golang.org/grpc"
+	"google.golang.org/grpc/codes"
+	"google.golang.org/grpc/status"
+
 	"github.com/thanos-io/thanos/pkg/component"
 	"github.com/thanos-io/thanos/pkg/store"
 	"github.com/thanos-io/thanos/pkg/store/storepb"
 	"github.com/thanos-io/thanos/pkg/testutil"
-	"google.golang.org/grpc"
-	"google.golang.org/grpc/codes"
-	"google.golang.org/grpc/status"
 )
 
 var testGRPCOpts = []grpc.DialOption{
@@ -27,10 +29,14 @@ var testGRPCOpts = []grpc.DialOption{
 }
 
 type testStore struct {
-	info storepb.InfoResponse
+	infoDelay time.Duration
+	info      storepb.InfoResponse
 }
 
 func (s *testStore) Info(ctx context.Context, r *storepb.InfoRequest) (*storepb.InfoResponse, error) {
+	if s.infoDelay > 0 {
+		time.Sleep(s.infoDelay)
+	}
 	return &s.info, nil
 }
 
@@ -54,6 +60,7 @@ type testStoreMeta struct {
 	extlsetFn        func(addr string) []storepb.LabelSet
 	storeType        component.StoreAPI
 	minTime, maxTime int64
+	infoDelay        time.Duration
 }
 
 type testStores struct {
@@ -82,6 +89,7 @@ func startTestStores(storeMetas []testStoreMeta) (*testStores, error) {
 				MaxTime:   meta.maxTime,
 				MinTime:   meta.minTime,
 			},
+			infoDelay: meta.infoDelay,
 		}
 		if meta.storeType != nil {
 			storeSrv.info.StoreType = meta.storeType.ToProto()
@@ -122,8 +130,6 @@ func (s *testStores) CloseOne(addr string) {
 }
 
 func TestStoreSet_Update(t *testing.T) {
-	defer leaktest.CheckTimeout(t, 10*time.Second)()
-
 	stores, err := startTestStores([]testStoreMeta{
 		{
 			storeType: component.Sidecar,
@@ -179,12 +185,17 @@ func TestStoreSet_Update(t *testing.T) {
 
 	// Testing if duplicates can cause weird results.
 	discoveredStoreAddr = append(discoveredStoreAddr, discoveredStoreAddr[0])
-	storeSet := NewStoreSet(nil, nil, func() (specs []StoreSpec) {
-		for _, addr := range discoveredStoreAddr {
-			specs = append(specs, NewGRPCStoreSpec(addr, false))
-		}
-		return specs
-	}, testGRPCOpts, time.Minute)
+	storeSet := NewStoreSet(nil, nil,
+		func() (specs []StoreSpec) {
+			for _, addr := range discoveredStoreAddr {
+				specs = append(specs, NewGRPCStoreSpec(addr, false))
+			}
+			return specs
+		},
+		func() (specs []RuleSpec) {
+			return nil
+		},
+		testGRPCOpts, time.Minute)
 	storeSet.gRPCInfoCallTimeout = 2 * time.Second
 	defer storeSet.Close()
 
@@ -461,7 +472,7 @@ func TestStoreSet_Update(t *testing.T) {
 
 	// Check stats.
 	expected = newStoreAPIStats()
-	expected[component.StoreAPI(nil)] = map[string]int{
+	expected[component.UnknownStoreAPI] = map[string]int{
 		"{l1=\"no-store-type\", l2=\"v3\"}": 1,
 	}
 	expected[component.Query] = map[string]int{
@@ -486,8 +497,6 @@ func TestStoreSet_Update(t *testing.T) {
 }
 
 func TestStoreSet_Update_NoneAvailable(t *testing.T) {
-	defer leaktest.CheckTimeout(t, 10*time.Second)()
-
 	st, err := startTestStores([]testStoreMeta{
 		{
 			extlsetFn: func(addr string) []storepb.LabelSet {
@@ -527,12 +536,15 @@ func TestStoreSet_Update_NoneAvailable(t *testing.T) {
 	st.CloseOne(initialStoreAddr[0])
 	st.CloseOne(initialStoreAddr[1])
 
-	storeSet := NewStoreSet(nil, nil, func() (specs []StoreSpec) {
-		for _, addr := range initialStoreAddr {
-			specs = append(specs, NewGRPCStoreSpec(addr, false))
-		}
-		return specs
-	}, testGRPCOpts, time.Minute)
+	storeSet := NewStoreSet(nil, nil,
+		func() (specs []StoreSpec) {
+			for _, addr := range initialStoreAddr {
+				specs = append(specs, NewGRPCStoreSpec(addr, false))
+			}
+			return specs
+		},
+		func() (specs []RuleSpec) { return nil },
+		testGRPCOpts, time.Minute)
 	storeSet.gRPCInfoCallTimeout = 2 * time.Second
 
 	// Should not matter how many of these we run.
@@ -548,8 +560,6 @@ func TestStoreSet_Update_NoneAvailable(t *testing.T) {
 
 // TestQuerierStrict tests what happens when the strict mode is enabled/disabled.
 func TestQuerierStrict(t *testing.T) {
-	defer leaktest.CheckTimeout(t, 5*time.Second)()
-
 	st, err := startTestStores([]testStoreMeta{
 		{
 			minTime: 12345,
@@ -585,6 +595,25 @@ func TestQuerierStrict(t *testing.T) {
 			},
 			storeType: component.Sidecar,
 		},
+		// Slow store.
+		{
+			minTime: 65644,
+			maxTime: 77777,
+			extlsetFn: func(addr string) []storepb.LabelSet {
+				return []storepb.LabelSet{
+					{
+						Labels: []storepb.Label{
+							{
+								Name:  "addr",
+								Value: addr,
+							},
+						},
+					},
+				}
+			},
+			storeType: component.Sidecar,
+			infoDelay: 2 * time.Second,
+		},
 	})
 
 	testutil.Ok(t, err)
@@ -595,14 +624,19 @@ func TestQuerierStrict(t *testing.T) {
 		return []StoreSpec{
 			NewGRPCStoreSpec(st.StoreAddresses()[0], true),
 			NewGRPCStoreSpec(st.StoreAddresses()[1], false),
+			NewGRPCStoreSpec(st.StoreAddresses()[2], true),
 		}
+	}, func() []RuleSpec {
+		return nil
 	}, testGRPCOpts, time.Minute)
 	defer storeSet.Close()
 	storeSet.gRPCInfoCallTimeout = 1 * time.Second
 
 	// Initial update.
 	storeSet.Update(context.Background())
-	testutil.Equals(t, 2, len(storeSet.stores), "two clients must be available for running store nodes")
+	testutil.Equals(t, 3, len(storeSet.stores), "three clients must be available for running store nodes")
+
+	testutil.Assert(t, storeSet.stores[st.StoreAddresses()[2]].cc.GetState().String() != "SHUTDOWN", "slow store's connection should not be closed")
 
 	// The store is statically defined + strict mode is enabled
 	// so its client + information must be retained.
@@ -619,8 +653,393 @@ func TestQuerierStrict(t *testing.T) {
 	storeSet.Update(context.Background())
 
 	// Check that the information is the same.
-	testutil.Equals(t, 1, len(storeSet.stores), "one client must remain available for a store node that is down")
+	testutil.Equals(t, 2, len(storeSet.stores), "two static clients must remain available")
 	testutil.Equals(t, curMin, storeSet.stores[staticStoreAddr].minTime, "minimum time reported by the store node is different")
 	testutil.Equals(t, curMax, storeSet.stores[staticStoreAddr].maxTime, "minimum time reported by the store node is different")
-	testutil.NotOk(t, storeSet.storeStatuses[staticStoreAddr].LastError)
+	testutil.NotOk(t, storeSet.storeStatuses[staticStoreAddr].LastError.originalErr)
+}
+
+func TestStoreSet_Update_Rules(t *testing.T) {
+	stores, err := startTestStores([]testStoreMeta{
+		{
+			extlsetFn: func(addr string) []storepb.LabelSet {
+				return []storepb.LabelSet{}
+			},
+			storeType: component.Sidecar,
+		},
+		{
+			extlsetFn: func(addr string) []storepb.LabelSet {
+				return []storepb.LabelSet{}
+			},
+			storeType: component.Rule,
+		},
+	})
+	testutil.Ok(t, err)
+	defer stores.Close()
+
+	for _, tc := range []struct {
+		name           string
+		storeSpecs     func() []StoreSpec
+		ruleSpecs      func() []RuleSpec
+		expectedStores int
+		expectedRules  int
+	}{
+		{
+			name: "stores, no rules",
+			storeSpecs: func() []StoreSpec {
+				return []StoreSpec{
+					NewGRPCStoreSpec(stores.orderAddrs[0], false),
+					NewGRPCStoreSpec(stores.orderAddrs[1], false),
+				}
+			},
+			expectedStores: 2,
+			expectedRules:  0,
+		},
+		{
+			name: "rules, no stores",
+			ruleSpecs: func() []RuleSpec {
+				return []RuleSpec{
+					NewGRPCStoreSpec(stores.orderAddrs[0], false),
+				}
+			},
+			expectedStores: 0,
+			expectedRules:  0,
+		},
+		{
+			name: "one store, different rule",
+			storeSpecs: func() []StoreSpec {
+				return []StoreSpec{
+					NewGRPCStoreSpec(stores.orderAddrs[0], false),
+				}
+			},
+			ruleSpecs: func() []RuleSpec {
+				return []RuleSpec{
+					NewGRPCStoreSpec(stores.orderAddrs[1], false),
+				}
+			},
+			expectedStores: 1,
+			expectedRules:  0,
+		},
+		{
+			name: "two stores, one rule",
+			storeSpecs: func() []StoreSpec {
+				return []StoreSpec{
+					NewGRPCStoreSpec(stores.orderAddrs[0], false),
+					NewGRPCStoreSpec(stores.orderAddrs[1], false),
+				}
+			},
+			ruleSpecs: func() []RuleSpec {
+				return []RuleSpec{
+					NewGRPCStoreSpec(stores.orderAddrs[0], false),
+				}
+			},
+			expectedStores: 2,
+			expectedRules:  1,
+		},
+		{
+			name: "two stores, two rules",
+			storeSpecs: func() []StoreSpec {
+				return []StoreSpec{
+					NewGRPCStoreSpec(stores.orderAddrs[0], false),
+					NewGRPCStoreSpec(stores.orderAddrs[1], false),
+				}
+			},
+			ruleSpecs: func() []RuleSpec {
+				return []RuleSpec{
+					NewGRPCStoreSpec(stores.orderAddrs[0], false),
+					NewGRPCStoreSpec(stores.orderAddrs[1], false),
+				}
+			},
+			expectedStores: 2,
+			expectedRules:  2,
+		},
+	} {
+		storeSet := NewStoreSet(nil, nil,
+			tc.storeSpecs,
+			tc.ruleSpecs,
+			testGRPCOpts, time.Minute)
+
+		t.Run(tc.name, func(t *testing.T) {
+			defer storeSet.Close()
+			storeSet.Update(context.Background())
+			testutil.Equals(t, tc.expectedStores, len(storeSet.stores))
+
+			gotRules := 0
+			for _, ref := range storeSet.stores {
+				if ref.HasRulesAPI() {
+					gotRules += 1
+				}
+			}
+
+			testutil.Equals(t, tc.expectedRules, gotRules)
+		})
+	}
+}
+
+func TestStoreSet_Rules_Discovery(t *testing.T) {
+	stores, err := startTestStores([]testStoreMeta{
+		{
+			extlsetFn: func(addr string) []storepb.LabelSet {
+				return []storepb.LabelSet{}
+			},
+			storeType: component.Sidecar,
+		},
+		{
+			extlsetFn: func(addr string) []storepb.LabelSet {
+				return []storepb.LabelSet{}
+			},
+			storeType: component.Rule,
+		},
+	})
+	testutil.Ok(t, err)
+	defer stores.Close()
+
+	type discoveryState struct {
+		name           string
+		storeSpecs     func() []StoreSpec
+		ruleSpecs      func() []RuleSpec
+		expectedStores int
+		expectedRules  int
+	}
+
+	for _, tc := range []struct {
+		states []discoveryState
+		name   string
+	}{
+		{
+			name: "StoreAPI and RulesAPI concurrent discovery",
+			states: []discoveryState{
+				{
+					name:           "no stores",
+					storeSpecs:     nil,
+					ruleSpecs:      nil,
+					expectedRules:  0,
+					expectedStores: 0,
+				},
+				{
+					name: "RulesAPI discovered",
+					storeSpecs: func() []StoreSpec {
+						return []StoreSpec{
+							NewGRPCStoreSpec(stores.orderAddrs[0], false),
+						}
+					},
+					ruleSpecs: func() []RuleSpec {
+						return []RuleSpec{
+							NewGRPCStoreSpec(stores.orderAddrs[0], false),
+						}
+					},
+					expectedRules:  1,
+					expectedStores: 1,
+				},
+			},
+		},
+
+		{
+			name: "StoreAPI discovery first, eventually discovered RulesAPI",
+			states: []discoveryState{
+				{
+					name:           "no stores",
+					storeSpecs:     nil,
+					ruleSpecs:      nil,
+					expectedRules:  0,
+					expectedStores: 0,
+				},
+				{
+					name: "StoreAPI discovered, no RulesAPI discovered",
+					storeSpecs: func() []StoreSpec {
+						return []StoreSpec{
+							NewGRPCStoreSpec(stores.orderAddrs[0], false),
+						}
+					},
+					expectedStores: 1,
+					expectedRules:  0,
+				},
+				{
+					name: "RulesAPI discovered",
+					storeSpecs: func() []StoreSpec {
+						return []StoreSpec{
+							NewGRPCStoreSpec(stores.orderAddrs[0], false),
+						}
+					},
+					ruleSpecs: func() []RuleSpec {
+						return []RuleSpec{
+							NewGRPCStoreSpec(stores.orderAddrs[0], false),
+						}
+					},
+					expectedStores: 1,
+					expectedRules:  1,
+				},
+			},
+		},
+
+		{
+			name: "RulesAPI discovery first, eventually discovered StoreAPI",
+			states: []discoveryState{
+				{
+					name:           "no stores",
+					storeSpecs:     nil,
+					ruleSpecs:      nil,
+					expectedRules:  0,
+					expectedStores: 0,
+				},
+				{
+					name:       "RulesAPI discovered, no StoreAPI discovered",
+					storeSpecs: nil,
+					ruleSpecs: func() []RuleSpec {
+						return []RuleSpec{
+							NewGRPCStoreSpec(stores.orderAddrs[0], false),
+						}
+					},
+					expectedStores: 0,
+					expectedRules:  0,
+				},
+				{
+					name: "StoreAPI discovered",
+					storeSpecs: func() []StoreSpec {
+						return []StoreSpec{
+							NewGRPCStoreSpec(stores.orderAddrs[0], false),
+						}
+					},
+					ruleSpecs: func() []RuleSpec {
+						return []RuleSpec{
+							NewGRPCStoreSpec(stores.orderAddrs[0], false),
+						}
+					},
+					expectedStores: 1,
+					expectedRules:  1,
+				},
+			},
+		},
+	} {
+		t.Run(tc.name, func(t *testing.T) {
+			currentState := 0
+
+			storeSet := NewStoreSet(nil, nil,
+				func() []StoreSpec {
+					if tc.states[currentState].storeSpecs == nil {
+						return nil
+					}
+
+					return tc.states[currentState].storeSpecs()
+				},
+				func() []RuleSpec {
+					if tc.states[currentState].ruleSpecs == nil {
+						return nil
+					}
+
+					return tc.states[currentState].ruleSpecs()
+				},
+				testGRPCOpts, time.Minute)
+
+			defer storeSet.Close()
+
+			for {
+				storeSet.Update(context.Background())
+				testutil.Equals(
+					t,
+					tc.states[currentState].expectedStores,
+					len(storeSet.stores),
+					"unexepected discovered stores in state %q",
+					tc.states[currentState].name,
+				)
+
+				gotRules := 0
+				for _, ref := range storeSet.stores {
+					if ref.HasRulesAPI() {
+						gotRules += 1
+					}
+				}
+				testutil.Equals(
+					t,
+					tc.states[currentState].expectedRules,
+					gotRules,
+					"unexpected discovered rules in state %q",
+					tc.states[currentState].name,
+				)
+
+				currentState = currentState + 1
+				if len(tc.states) == currentState {
+					break
+				}
+			}
+		})
+	}
+}
+
+type errThatMarshalsToEmptyDict struct {
+	msg string
+}
+
+// MarshalJSON marshals the error and returns and empty dict, not the error string.
+func (e *errThatMarshalsToEmptyDict) MarshalJSON() ([]byte, error) {
+	return json.Marshal(map[string]string{})
+}
+
+// Error returns the original, underlying string.
+func (e *errThatMarshalsToEmptyDict) Error() string {
+	return e.msg
+}
+
+// Test highlights that without wrapping the error, it is marshaled to empty dict {}, not its message.
+func TestStringError(t *testing.T) {
+	dictErr := &errThatMarshalsToEmptyDict{msg: "Error message"}
+	stringErr := &stringError{originalErr: dictErr}
+
+	storestatusMock := map[string]error{}
+	storestatusMock["dictErr"] = dictErr
+	storestatusMock["stringErr"] = stringErr
+
+	b, err := json.Marshal(storestatusMock)
+
+	testutil.Ok(t, err)
+	testutil.Equals(t, []byte(`{"dictErr":{},"stringErr":"Error message"}`), b, "expected to get proper results")
+}
+
+// Errors that usually marshal to empty dict should return the original error string.
+func TestUpdateStoreStateLastError(t *testing.T) {
+	tcs := []struct {
+		InputError      error
+		ExpectedLastErr string
+	}{
+		{errors.New("normal_err"), `"normal_err"`},
+		{nil, `null`},
+		{&errThatMarshalsToEmptyDict{"the error message"}, `"the error message"`},
+	}
+
+	for _, tc := range tcs {
+		mockStoreSet := &StoreSet{
+			storeStatuses: map[string]*StoreStatus{},
+		}
+		mockStoreRef := &storeRef{
+			addr: "testStore",
+		}
+
+		mockStoreSet.updateStoreStatus(mockStoreRef, tc.InputError)
+
+		b, err := json.Marshal(mockStoreSet.storeStatuses["testStore"].LastError)
+		testutil.Ok(t, err)
+		testutil.Equals(t, tc.ExpectedLastErr, string(b))
+	}
+}
+
+func TestUpdateStoreStateForgetsPreviousErrors(t *testing.T) {
+	mockStoreSet := &StoreSet{
+		storeStatuses: map[string]*StoreStatus{},
+	}
+	mockStoreRef := &storeRef{
+		addr: "testStore",
+	}
+
+	mockStoreSet.updateStoreStatus(mockStoreRef, errors.New("test err"))
+
+	b, err := json.Marshal(mockStoreSet.storeStatuses["testStore"].LastError)
+	testutil.Ok(t, err)
+	testutil.Equals(t, `"test err"`, string(b))
+
+	// updating status without and error should clear the previous one.
+	mockStoreSet.updateStoreStatus(mockStoreRef, nil)
+
+	b, err = json.Marshal(mockStoreSet.storeStatuses["testStore"].LastError)
+	testutil.Ok(t, err)
+	testutil.Equals(t, `null`, string(b))
 }
